<?php

/**
 * Handles the events and options for the WP eCommerce Plugin.
 *
 * @package Mediaburst Ecommerce SMS
 * @since Alpha 1
 */
class MBESMS_WPeCommerce extends MBESMS_Generic {

	// HOOKS
	// =====
	
	/**
	 * Initiate!
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function __construct() {
		$this->setup( 'mbesms-wpsc', 'plugin' );
		parent::__construct();
		if ( is_admin() ) {
			$this->add_filter( 'wpsc_settings_redirect_url', 'settings_redirect_url' );
			$this->add_action( 'load-dashboard_page_wpsc-sales-logs', 'load_sales_log' );
		}
		$this->add_action( 'wpsc_admin_settings_page', 'admin_settings_page', 0 );
		$this->add_action( 'wpsc_confirm_checkout' );
		$this->add_action( 'wpsc_edit_order_status' );
		$this->add_filter( 'wpsc_add_unique_names' );
		$this->add_action( 'wpsc_billing_details_bottom' );
		// $this->add_action( 'init' );
		$this->version = 2;
	}

	/**
	 * Hooks the WPSC filter wpsc_settings_redirect_url in order to
	 * process the options we've added.
	 *
	 * @param string $url A URL we must return unharmed and unaltered 
	 * @return string The URL we must return unharmed and unaltered 
	 **/
	public function settings_redirect_url( $url ) {
		$this->process_settings();
		return $url;
	}

	/**
	 * Hooks the WP Ecommerce wpsc_admin_settings_page action to
	 * add the custom message specification fields to the 
	 * Settings > Store > Admin area.
	 *
	 * @return void
	 **/
	public function admin_settings_page() {
		global $wpsc_purchlog_statuses;
		$vars = array();
		$vars[ 'admin_message' ]		= $this->get_option( 'admin_message' );
		$vars[ 'admin_number' ]			= $this->get_option( 'admin_number' );
		$vars[ 'from' ]					= $this->get_option( 'from' );
		$vars[ 'selected_statuses' ]	= $this->get_option( 'selected_statuses', array() );
		$vars[ 'send_admin_message' ]	= $this->get_option( 'send_admin_message' );
		$vars[ 'send_status_change' ]	= $this->get_option( 'send_status_change' );
		$vars[ 'statuses' ] 			= $wpsc_purchlog_statuses;
		$vars[ 'status_change' ] 		= $this->get_option( 'status_change' );
		// @TODO: Add in JS character count
		$this->render_admin( 'wp-ecommerce-messages.php', $vars );
	}

	/**
	 * Hooks the WP eCommerce wpsc_add_unique_names filter to
	 * add in a new field type for the SMS update number.
	 *
	 * @param array $unique_names Names 
	 * @return array Names
	 **/
	public function wpsc_add_unique_names( $unique_names ) {
		$unique_names[] = 'billingsmsupdates';
		sort( $unique_names );
		return $unique_names;
	}

	/**
	 * Hooks the WP wpsc_edit_order_status action 
	 * 
	 * Hook a change in status from an existing order.
	 *
	 * @param array $purchase_data Some purchase data 
	 * @return void
	 **/
	public function wpsc_edit_order_status( $purchase_data ) {
		// The admin could have blocked sending of these messages
		if ( ! $this->get_option( 'send_status_change' ) )
			return;
		// Check a message should be sent for this status
		$selected_statuses = $this->get_option( 'selected_statuses', array() );
		if ( ! in_array( $purchase_data[ 'new_status' ], $selected_statuses ) )
			return;
		// What it says near this hook in the WPSC code:
		// "in the future when everyone is using the 2.0 merchant api, we should use the merchant class to update the staus,
		//  then you can get rid of this hook and have each person overwrite the method that updates the status."
		// However see: http://getshopped.org/forums/topic/is-the-wpsc_edit_order_status-action-going-away/
		$log_id = (int) $purchase_data[ 'purchlog_id' ];

		if ( ! ( $mobile_number = $this->get_purchase_mobile( $log_id ) ) ) {
			return;
		}

		$country = $this->get_purchase_country( $log_id );
		// No country on the purchase, assume the base country for the store
		if ( ! $country )
			$country = get_option( 'base_country' );
			
		$mobile_number = $this->format_mobile_number( $mobile_number, $country );

		// Without a mobile number we are lost!
		if ( ! $mobile_number )
			return;

		$message = $this->get_status_change_msg( $purchase_data );
		
		$delivery_receipt = array(
			'PLUGIN' => 'mbesms_wpecommerce',
			'LOG_ID' => $log_id,
		);
		// error_log( "Send SMS $mobile_number, $message, $delivery_receipt" );
		$result = $this->send_sms( $mobile_number, $message, $delivery_receipt );
		if ( false !== $result ) {
			$this->record_sms( $log_id, $mobile_number, $message, $result );
		} 
	}

	/**
	 * Hooks the WP wpsc_confirm_checkout action 
	 * 
	 * Hook a new order in the shop.
	 *
	 * @param int $log_id The ID for this purchase log 
	 * @return void
	 **/
	public function wpsc_confirm_checkout( $log_id ) {
		global $wpdb;
		$sql = " SELECT * FROM {$wpdb->prefix}wpsc_purchase_logs WHERE id = %d ";
		$purchase_data = array( 'purchlog_id' => $log_id );
		$purchase_data[ 'purchlog_data' ] = $wpdb->get_row( $wpdb->prepare( $sql, $log_id ), ARRAY_A );
		// This hook seems to get fired twice. Which is odd. Avoid it the second time.
		if ( 2 != $purchase_data[ 'purchlog_data' ][ 'processed' ] )
			return;
		$message = $this->get_admin_new_order_msg( $purchase_data );
		$mobile_number = $this->get_option( 'admin_number' );
		$this->send_sms( $mobile_number, $message );
	}

	/**
	 * Hooks the WPSC action wpsc_billing_details_bottom to add our SMS
	 * notifications for the admin.
	 *
	 * @return void
	 **/
	public function wpsc_billing_details_bottom() {
		global $wpdb;
		$wpsc_meta = WPSC_TABLE_META;
		$sql = " SELECT sms.meta_key AS client_id, sms.meta_value AS sms_details FROM $wpsc_meta AS sms WHERE sms.object_type = 'mbsms' AND sms.object_id = %d ORDER BY sms.meta_id DESC ";
		$log_id = (int) wpsc_purchaselog_details_purchnumber();
		$prepared_sql = $wpdb->prepare( $sql, $log_id );
		$notifications = $wpdb->get_results( $prepared_sql, ARRAY_A );
		foreach ( $notifications as & $n ) {
			$sms_details = maybe_unserialize( $n[ 'sms_details' ] );
			$sql = " SELECT meta_value as dlr_details FROM $wpsc_meta AS dlr WHERE meta_key = %s AND object_type = 'mbdlr' AND object_id = %d ORDER BY meta_id DESC ";
			$prepared_sql = $wpdb->prepare( $sql, $sms_details[ 'result' ][ 'clientid' ], $log_id );
			$dlr_details = $wpdb->get_var( $prepared_sql );
			if ( ! is_null( $dlr_details ) )
				$n[ 'dlr_details' ] = maybe_unserialize( $dlr_details );
			else
				$n[ 'dlr_details' ] = array();
			$n = (object) array_merge( $n, $sms_details, $n[ 'dlr_details' ] );
			if ( ! isset( $n->status ) ) {
				$n->status_date = current_time( 'timestamp' );
				$n->status = 'NOT YET KNOWN';
			}
			if ( isset( $n->success ) && ! $n->success )
				$n->error_message = $n->result[ 'error_message' ];
			else
				$n->success = true;
			unset( $n->sms_details );
			unset( $n->dlr_details );
		}
		$mobile_number = $this->get_purchase_mobile( $log_id );
		$country_isocode = $this->get_purchase_country( $log_id );
		$mobile_number = $this->format_mobile_number( $mobile_number, $country_isocode );
		$vars = array();
		$vars[ 'notifications' ] = $notifications;
		$vars[ 'mobile_number' ] = $mobile_number;
		$this->render_admin( 'wp-ecommerce-notifications.php', $vars );
	}

	/**
	 * Hooks the WP load-dashboard_page_wpsc-sales-logs action.load_sales_log
	 *
	 * @return void
	 **/
	public function load_sales_log() {
		if ( ! @ $_POST[ '_mbesms_nonce' ] )
			return;
		$log_id = (int) @ $_GET[ 'purchaselog_id' ];
		check_admin_referer( "save_mobile_$log_id", '_mbesms_nonce' );
		$billingsmsupdates = $_POST[ 'billingsmsupdates' ];
		$this->set_checkout_field( $log_id, 'billingsmsupdates', $billingsmsupdates );
	}

	// METHODS
	// =======

	/**
	 * Record the delivery receipt POSTed details in the WPSC 
	 * generic meta table.
	 *
	 * @return void
	 **/
	public function handle_delivery_receipt() {
	    $client_id = @ $_POST[ 'CLIENT_ID' ];
	    $status = @ $_POST[ 'STATUS' ];
	    $dest_addr = @ $_POST[ 'DEST_ADDR' ];
	    $err_code = (int) @ $_POST[ 'ERR_CODE' ];
	    $msg_id = @ $_POST[ 'MSG_ID' ];
	    $src_addr = @ $_POST[ 'SRC_ADDR' ];
	    $submit_date = @ $_POST[ 'SUBMIT_DATE' ]; // UTC?
	    $status_date = @ $_POST[ 'STATUS_DATE' ]; // UTC?
	    $log_id = @ $_POST[ 'LOG_ID' ];

		$submit_timestamp = $this->mediaburst_date_to_timestamp( $submit_date, true );
		$status_timestamp = $this->mediaburst_date_to_timestamp( $status_date, true );

		$meta_value = array(
			'err_code' => $err_code,
			'mobile_number' => $dest_addr,
			'msg_id' => $msg_id,
			'status' => $status,
			'status_date' => $status_timestamp,
			'submit_date' => $submit_timestamp,
		);
		$meta_key = $client_id;
		wpsc_update_meta( $log_id, $meta_key, $meta_value, 'mbdlr' );
	}

	/**
	 * Record the details of an SMS in the WPSC generic meta table.
	 *
	 * @param int $log_id The purchase log ID to associate this notification with
	 * @param string $mobile_number The mobile number the message was sent to
	 * @param string $message The message which was sent
	 * @param array $result The reply from the Mediaburst SMS API 
	 * @return void
	 **/
	protected function record_sms( $log_id, $mobile_number, $message, $result ) {
		$meta_value = array(
			'message' => $message,
			'mobile_number' => $mobile_number,
			'result' => $result[ 0 ],
			'submit_date' => current_time( 'timestamp' ),
			'success' => $result[ 0 ][ 'success' ],
		);
		// Key the meta on the clientID to make it easy/possible to retrieve later
		$meta_key = $result[ 0 ][ 'clientid' ];
		wpsc_update_meta( $log_id, $meta_key, $meta_value, 'mbsms' );
	}

	/**
	 * Get the template for the status change message, replace all tokens,
	 * return the final message.
	 *
	 * @param array $purchase_data The purchase data passed to the wpsc_edit_order_status filter 
	 * @return string The message
	 **/
	protected function get_status_change_msg( $purchase_data ) {
		$log_id = (int) $purchase_data[ 'purchlog_id' ];
		$message_tpl = $this->get_option( 'status_change' );
		$currency_args = array( 
			'display_as_html' => false, // No HTML characters
			'display_currency_symbol' => false, // Don't use the currency symbol, SMS charset may not cope
		);
		$search = array(
			'%purchase_id%',
			'%shop_name%',
			'%total_price%',
			'%order_status%'
		);
		$replace = array(
			$log_id,
			get_option( 'blogname' ),
			wpsc_currency_display( $purchase_data[ 'purchlog_data' ][ 'totalprice' ], $currency_args ),
			$this->get_purchase_status( $purchase_data[ 'new_status' ] ),
		);
		return str_replace( $search, $replace, $message_tpl );
	}

	/**
	 * Get the template for the status change message, replace all tokens,
	 * return the final message.
	 *
	 * @param array $purchase_data The purchase data passed to the wpsc_edit_order_status filter 
	 * @return string The message
	 **/
	protected function get_admin_new_order_msg( $purchase_data ) {
		$log_id = (int) $purchase_data[ 'purchlog_id' ];
		$message_tpl = $this->get_option( 'admin_message' );
		$currency_args = array( 
			'display_as_html' => false, // No HTML characters
			'display_currency_symbol' => false, // Don't use the currency symbol, SMS charset may not cope
		);
		$search = array(
			'%purchase_id%',
			'%shop_name%',
			'%total_price%',
		);
		$replace = array(
			$log_id,
			get_option( 'blogname' ),
			wpsc_currency_display( $purchase_data[ 'purchlog_data' ][ 'totalprice' ], $currency_args ),
		);
		return str_replace( $search, $replace, $message_tpl );
	}

	/**
	 * Return the mobile number, as entered by the customer, from a particular order.
	 *
	 * @param int $log_id The ID for the purchase log we want the mobile number for 
	 * @return string The mobile number as entered by the customer
	 **/
	protected function get_purchase_mobile( $log_id ) {
		return $this->get_checkout_field( $log_id, 'billingsmsupdates' );
	}

	/**
	 * Return the mobile number, as entered by the customer, from a particular order.
	 *
	 * @param int $log_id The ID for the purchase log we want the mobile number for 
	 * @return string The mobile number as entered by the customer
	 **/
	protected function get_purchase_country( $log_id ) {
		return $this->get_checkout_field( $log_id, 'billingcountry' );
	}

	/**
	 * Return a checkout field, identified by a unique_name, from a particular purchase.
	 *
	 * @param int $log_id The ID for the purchase log we want the field for 
	 * @return string The field value, as entered by the customer
	 **/
	protected function get_checkout_field( $log_id, $unique_name ) {
		global $wpdb;
		$checkout_forms = WPSC_TABLE_CHECKOUT_FORMS;
		$submitted_form_data = WPSC_TABLE_SUBMITED_FORM_DATA;
		$sql = " SELECT value FROM $submitted_form_data WHERE log_id = %d AND form_id IN ( SELECT id FROM $checkout_forms WHERE unique_name = %s ) ";
		// error_log( "SQL: " . $wpdb->prepare( $sql, $log_id, $unique_name ) );
		return $wpdb->get_var( $wpdb->prepare( $sql, $log_id, $unique_name ) );
	}

	/**
	 * Return a checkout field, identified by a unique_name, from a particular purchase.
	 *
	 * @param int $log_id The ID for the purchase log we want the field for 
	 * @return string The field value, as entered by the customer
	 **/
	protected function set_checkout_field( $log_id, $unique_name, $value ) {
		global $wpdb;
		$checkout_forms = WPSC_TABLE_CHECKOUT_FORMS;
		$submitted_form_data = WPSC_TABLE_SUBMITED_FORM_DATA;
		$sql = " UPDATE $submitted_form_data SET value = %s WHERE log_id = %d AND form_id IN ( SELECT id FROM $checkout_forms WHERE unique_name = %s ) ";
		$prepared_sql = $wpdb->prepare( $sql, $value, $log_id, $unique_name );
		$wpdb->query( $prepared_sql );
	}

	/**
	 * Return the Purchase Status name when passed an order number.
	 *
	 * @param int $order The order value for this status 
	 * @return void
	 **/
	protected function get_purchase_status( $order ) {
		global $wpsc_purchlog_statuses;
		foreach ( $wpsc_purchlog_statuses as $status ) {
			if ( $order == $status[ 'order' ] )
				return $status[ 'label' ];
		}
	}
	
	/**
	 * Handle the POSTed data for our settings.
	 *
	 * @return void
	 **/
	protected function process_settings() {
		$send_admin_message = (bool) @ $_POST[ 'send_admin_message' ];
		$admin_number = @ $_POST[ 'admin_number' ];
		$admin_message =  @ $_POST[ 'admin_message' ];
		$send_status_change = (bool) @ $_POST[ 'send_status_change' ];
		$status_change = @ $_POST[ 'status_change' ];
		$selected_statuses = (array) @ $_POST[ 'selected_statuses' ];
		$from = @ $_POST[ 'from' ];

		// Do we trust user input? Hell no.
		$selected_statuses = array_map( 'absint', $selected_statuses );
		$from = $this->sanitise_from_str( $from );
		
		$this->update_option( 'send_admin_message', $send_admin_message );
		$this->update_option( 'admin_number', $admin_number );
		$this->update_option( 'admin_message', $admin_message );
		$this->update_option( 'send_status_change', $send_status_change );
		$this->update_option( 'status_change', $status_change );
		$this->update_option( 'selected_statuses', $selected_statuses );
		$this->update_option( 'from', $from );
	}

	/**
	 * Checks the DB structure is up to date.
	 *
	 * @return void
	 **/
	protected function maybe_update() {
		global $wpdb;
		$option_name = 'mbesms-wpsc-version';
		$version = get_option( $option_name, 0 );

		if ( $this->version == $version )
			return;

		if ( $version < 1 ) {
			$admin_message = __( 'You have a new order at %shop_name%.', 'mbesms' );
			$status_change = __( 'Your order at %shop_name% is now: %order_status%.', 'mbesms' );
			$this->update_option( 'send_admin_message', 1 );
			$this->update_option( 'admin_message', $admin_message );
			$this->update_option( 'admin_number', '' );
			$this->update_option( 'send_status_change', 1 );
			$this->update_option( 'status_change', $status_change );
			$this->update_option( 'selected_statuses', array( 1, 2, 3, 4, 5, 6 ) );
			error_log( "MBES WPSC: Set default messages" );
		}
		
		if ( $version < 2 ) {
			$checkout_forms = WPSC_TABLE_CHECKOUT_FORMS;
			$sql = " SELECT COUNT( * ) FROM $checkout_forms WHERE unique_name = 'billingsmsupdates' AND checkout_set = 0 AND active = 1 ";
			if ( ! $wpdb->get_var( $sql ) ) {
				$sql = " INSERT INTO $checkout_forms ( `name`, `type`, `mandatory`, `display_log`, `default`, `active`, `checkout_order`, `unique_name`) VALUES ( '%s', 'heading', '0', '0', '', '1', 49,''),( '%s', 'text', '0', '0', '', '1', 50,'billingsmsupdates');";
				$prepared_sql = $wpdb->prepare( $sql, __( "We'd like to send you SMS updates on your order", 'mbesms' ), __( 'Mobile Phone', 'mbesms' ) );
				$wpdb->query( $prepared_sql );
				error_log( "MBES WPSC: Added fields" );
			}
			
		}

		error_log( "MBES WPSC: Done updates" );
		update_option( $option_name, $this->version );
	}

} // END MBESMS_WPeCommerce class 

$mbesms_wpecommerce = new MBESMS_WPeCommerce();

?>